/* xcard.c 
 * io code for the xradiocardprogram 
 * courtesy Gideon le Grange 
 *
 * this is the code that does the actual talking to the card 
 *
 * July 29th 1995
 *
 * With thanks to Gideon, I've used lots of his code for my
 * xradiocard program. See the README file for complete
 * acknowledgements which are richly deserved.
 *
 * V.1.1 March '96
 *
 * Added stuff for tuning indicator and muting
 * Some simplification of code by just doing byte I/O. No need
 * to use an offset from the card's base I/O port, It seems they're
 * all the same, which probably saved a few transistors in the
 * hardware.
 *
 *          Frans Brinkman
 *
 * V.2.0 March '97
 *
 * Modified to suite also Zoltrix card, which use a different protocol
 * to set frequency and volume. I choosed to mix in a single program
 * instead of having two programs around for two different cards.
 *
 *          Romolo Manfredini
 */

#include <asm/io.h>
#include <unistd.h>

extern int muted;
extern int lastvol;

unsigned long RadioBase;	/* base port for RadioTrack */
unsigned long RadioType;	/* Radio card type (RadioTrack/Zoltrix) */

#define FCODE(f)	((unsigned long long)(((float)(f)-88.0)*200))+0x4d1c

/*
   data sent in sequence to the card (Zoltrix):

   an init sequence,
   45 bit programming datas so composed:
   1100010010ffffffff1FFFFFFFF1001100s001000000001
   where ffffffff is the LSByte of the frequency
   FFFFFFFF is the MSByte of the frequency
   s        is the stereo flag (1 on, 0 off)
   and a ending sequence.

   The encoding of 1 and 0 seem to vary:
   The dos program supplied with my card (radiodos.exe),  uses
   1=b3b 0=f7f when launched from command line but 808 and c4c
   when used interactively (thanks to the author of dosemu for
   giving me (with a little patch)  the capability of snooping
   the io ports activity).
   Also init and ending sequence have a different encoding.

   My program at the moment use only interactive encoding.
   I will try to study longer the meaning of the bits.
 */

void 
CardOff (void)
{
  int l;
  /* switch card off */
  outb (0, RadioBase);
  outb (0, RadioBase);
  l = inb (RadioBase + 3);	/* Zoltrix card requires to be readed */
}

int 
CardMute (void)
{
  int l;
  /* return 1 if now muted, 0 if muting now off */

  if (!muted)			/* case we weren't muted */
    {
      /* make it shut up */
      /* switch card off */
      outb (0, RadioBase);
      outb (0, RadioBase);
      l = inb (RadioBase + 3);
      return (1);
    }
  else
    {
      /* switch sound back on */
      outb (0, RadioBase);
      if (RadioType == 1)
	{
	  outb (lastvol, RadioBase);
	  usleep (20000);
	  l = inb (RadioBase + 2);
	}
      else
	{
	  outb (0xc8, RadioBase);
	}
      return (0);
    }
}

void 
VolumeDown (void)
{
  int l;
  if (!muted)
    {
      /* vol down a notch */
      if (RadioType == 1)
	{
	  outb (lastvol, RadioBase);
	  usleep (20000);
	  l = inb (RadioBase + 2);
	}
      else
	{
	  outb (0x48, RadioBase);
	  usleep (10000);
	  outb (0xc8, RadioBase);
	}
    }
  else
    {
      outb (0, RadioBase);
      if (RadioType == 1)
	{
	  outb (lastvol, RadioBase);
	  usleep (20000);
	  l = inb (RadioBase + 2);
	}
      else
	{
	  outb (0xc8, RadioBase);
	}
    }
}

void 
VolumeUp (void)
{
  int l;
  if (!muted)
    {
      /* vol up a notch */
      if (RadioType == 1)
	{
	  outb (lastvol, RadioBase);
	  usleep (20000);
	  l = inb (RadioBase + 2);
	}
      else
	{
	  outb (0x88, RadioBase);
	  usleep (10000);
	  outb (0xc8, RadioBase);
	}
    }
  else
    {
      outb (0, RadioBase);
      if (RadioType == 1)
	{
	  outb (lastvol, RadioBase);
	  usleep (20000);
	  l = inb (RadioBase + 2);
	}
      else
	{
	  outb (0xc8, RadioBase);
	}
    }
}

void 
Tune (float freq, unsigned int stereo)
{
  /* tunes the radio to the desired frequency */
  /* as usual in this program, two implementations depending on the card */

  unsigned long int bits;
  unsigned long long bitmask, f;
  int i, mask;

  if (RadioType == 0)
    {
      bits = (freq * 40) + 10486188;
      /* yes, i know. now where does THAT come from? 
       * working out that bits=(freq*multiplier)+base 
       * was hard, and working out the base and multiplier
       * was tricky. */
      mask = 1;
      for (i = 0; i < 24; ++i)
	{
	  if ((bits & mask) == 0)
	    {
	      /* 0 bit */
	      outb (0x1, RadioBase);
	      outb (0x1, RadioBase);
	      outb (0x3, RadioBase);
	      outb (0x3, RadioBase);
	    }
	  else
	    {
	      /* bit is 1 */
	      outb (0x5, RadioBase);
	      outb (0x5, RadioBase);
	      outb (0x7, RadioBase);
	      outb (0x7, RadioBase);
	    }
	  mask = mask * 2;
	}
      /* termination sequence */
      outb (0x0, RadioBase);	/* keeps current volume setting */
      usleep (10000);
      outb (0xc8, RadioBase);
    }
  else
    {
      f = FCODE (freq);
      bitmask = 0xc480402c10080000ull;
      i = 45;
      outb (0x00, RadioBase);
      outb (0x00, RadioBase);
      outb (0x40, RadioBase);
      outb (0xc0, RadioBase);
      bitmask = (bitmask ^ ((f & 0xff) << 47) ^ ((f & 0xff00) << 30) ^ (stereo << 31));
      while (i--)
	{
	  if ((bitmask & 0x8000000000000000ull) != 0)
	    {
	      outb (0x80, RadioBase);
	      outb (0x00, RadioBase);
	      outb (0x80, RadioBase);
	    }
	  else
	    {
	      outb (0xc0, RadioBase);
	      outb (0x40, RadioBase);
	      outb (0xc0, RadioBase);
	    }
	  bitmask *= 2;
	}
      /* termination sequence */
      outb (0x80, RadioBase);
      outb (0xc0, RadioBase);
      outb (0x40, RadioBase);
      outb (0x00, RadioBase);
      usleep (20000);
      i = inb (RadioBase + 2);
      usleep (20000);
      i = inb (RadioBase + 1);
      outb (lastvol, RadioBase);
      usleep (20000);
      i = inb (RadioBase + 2);
    }
}

int 
IsTuned (void)
{
  int x1,x2;

/* test tuning */
/*
   DO NOT CHANGE THE TIMING !?!?!?

   Scanning seems to be pretty unreliable, or at least using
   the one card I've got. A shorter wait will miss lots of valid
   stations, longer ones will make it find lots of noisy signals.
   Should anyone ever find out how it is supposed to work please
   let me know. For absolutely no reason that I can see it also seems
   to work slightly better if the frequencies used are multiples
   of .05, but that might just be coincidence.
   
   I added a two reading test to improve performances. (Only for Zoltrix)
   Romolo Manfredini
 */
  if (RadioType == 1)
    {
      outb (0x00, RadioBase);
      usleep (1000000);		/* any faster and it won't work? */
      x1 = inb (RadioBase+2);
    }
  else
    {
      outb (0xf8, RadioBase);
      usleep (200000);		/* any faster and it won't work? */
      x1 = inb (RadioBase);
    }
  x2=0;
  if (RadioType == 1)
    {
      usleep (200000);
      x2 =inb(RadioBase+1);
      outb (0x00, RadioBase);
    }
  else
    {
      outb (0xe8, RadioBase);
    }

  /* if (x1||x2) == ff not tuned, if x == fd then tuned */

  if (((x1 | x2)  & 255) != 0x0ff)
    return (1);
  else
    return (0);
}
